@@ -7,16 +7,19 @@ import curses
 import time
 import getopt
 import sys
+import importlib
 
 # Modules:
 from calcure.calendars import Calendar
 from calcure.configuration import cf
 from calcure.weather import Weather
-from calcure.repository import Importer, FileRepository
+from calcure.repository import FileRepository
+from calcure.importers import Importer
 from calcure.dialogues import clear_line
 from calcure.screen import Screen
 from calcure.data import *
 from calcure.controls import *
+from calcure.helpers import initialize_colors
 
 
 # Language:
@@ -28,50 +31,15 @@ elif cf.LANG == "it":
     from calcure.translations.it import *
 elif cf.LANG == "br":
     from calcure.translations.br import *
+elif cf.LANG == "tr":
+    from calcure.translations.tr import *
+elif cf.LANG == "zh":
+    from calcure.translations.zh import *
 else:
     from calcure.translations.en import *
 
 
-__version__ = "2.5.5"
-
-
-def initialize_colors():
-    """Define all the color pairs"""
-    curses.start_color()
-    curses.use_default_colors()
-    curses.init_pair(Color.DAY_NAMES.value, cf.COLOR_DAY_NAMES, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.WEEKENDS.value, cf.COLOR_WEEKENDS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.HINTS.value, cf.COLOR_HINTS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.TODAY.value, cf.COLOR_TODAY, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.DAYS.value, cf.COLOR_DAYS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.WEEKEND_NAMES.value, cf.COLOR_WEEKEND_NAMES, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.BIRTHDAYS.value, cf.COLOR_BIRTHDAYS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.PROMPTS.value, cf.COLOR_PROMPTS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.CONFIRMATIONS.value, cf.COLOR_CONFIRMATIONS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.TITLE.value, cf.COLOR_TITLE, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.TODO.value, cf.COLOR_TODO, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.DONE.value, cf.COLOR_DONE, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.IMPORTANT.value, cf.COLOR_IMPORTANT, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.TIMER.value, cf.COLOR_TIMER, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.TIMER_PAUSED.value, cf.COLOR_TIMER_PAUSED, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.HOLIDAYS.value, cf.COLOR_HOLIDAYS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.EVENTS.value, cf.COLOR_EVENTS, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.TIME.value, cf.COLOR_TIME, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.WEATHER.value, cf.COLOR_WEATHER, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.UNIMPORTANT.value, cf.COLOR_UNIMPORTANT, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.CALENDAR_HEADER.value, cf.COLOR_CALENDAR_HEADER, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.ACTIVE_PANE.value, cf.COLOR_ACTIVE_PANE, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.SEPARATOR.value, cf.COLOR_SEPARATOR, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.EMPTY.value, cf.COLOR_BACKGROUND, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.CALENDAR_BOARDER.value, cf.COLOR_CALENDAR_BOARDER, cf.COLOR_BACKGROUND)
-    curses.init_pair(Color.DEADLINES.value, cf.COLOR_DEADLINES, cf.COLOR_BACKGROUND)
-
-    if not cf.MINIMAL_WEEKEND_INDICATOR:
-        curses.init_pair(Color.WEEKENDS.value, curses.COLOR_BLACK, cf.COLOR_WEEKENDS)
-    if not cf.MINIMAL_TODAY_INDICATOR:
-        curses.init_pair(Color.TODAY.value, curses.COLOR_BLACK, cf.COLOR_TODAY)
-    if not cf.MINIMAL_DAYS_INDICATOR:
-        curses.init_pair(Color.DAYS.value, curses.COLOR_BLACK, cf.COLOR_DAYS)
+__version__ = "2.7.1"
 
 
 def read_items_from_user_arguments(screen, user_tasks, user_events, file_repository):
@@ -238,14 +206,15 @@ class TimerView(View):
 class JournalView(View):
     """Displays a list of all tasks"""
 
-    def __init__(self, stdscr, y, x, user_tasks, screen):
+    def __init__(self, stdscr, y, x, user_tasks, user_ics_tasks, screen):
         super().__init__(stdscr, y, x)
         self.user_tasks = user_tasks
+        self.user_ics_tasks = user_ics_tasks
         self.screen = screen
 
     def render(self):
         """Render the list of tasks"""
-        if not self.user_tasks.items and cf.SHOW_NOTHING_PLANNED:
+        if not self.user_tasks.items and not self.user_ics_tasks.items and cf.SHOW_NOTHING_PLANNED:
             self.display_line(self.y, self.x, MSG_TS_NOTHING, Color.UNIMPORTANT)
         for index, task in enumerate(self.user_tasks.items):
             task_view = TaskView(self.stdscr, self.y, self.x, task, self.screen)
@@ -254,6 +223,12 @@ class JournalView(View):
                 self.display_line(self.y, self.x, str(index + 1), Color.TODAY)
             self.y += 1
 
+        self.y += 1
+        for index, task in enumerate(self.user_ics_tasks.items):
+            task_view = TaskView(self.stdscr, self.y, self.x, task, self.screen)
+            task_view.render()
+            self.y += 1
+
 
 class EventView(View):
     """Parent class to display events"""
@@ -396,13 +371,16 @@ class DeadlineView(EventView):
 class DailyView(View):
     """Display all events occurring on this days"""
 
-    def __init__(self, stdscr, y, x, repeated_user_events, user_events, holidays, birthdays, user_tasks, screen, index_offset, is_selection_day=True):
+    def __init__(self, stdscr, y, x, repeated_user_events, user_events, user_ics_events, holidays,
+                                birthdays, user_tasks, user_ics_tasks, screen, index_offset, is_selection_day=True):
         super().__init__(stdscr, y, x)
         self.repeated_user_events = repeated_user_events.filter_events_that_day(screen)
         self.user_events = user_events.filter_events_that_day(screen)
+        self.user_ics_events = user_ics_events.filter_events_that_day(screen)
         self.holidays = holidays.filter_events_that_day(screen)
         self.birthdays = birthdays.filter_events_that_day(screen)
         self.deadlines = user_tasks.filter_events_that_day(screen)
+        self.deadlines_ics = user_ics_tasks.filter_events_that_day(screen)
         self.screen = screen
         self.index_offset = index_offset
         self.y_cell = (self.screen.y_max - 3) // 6
@@ -426,23 +404,25 @@ class DailyView(View):
                 self.display_line(self.y + self.y_cell - 2, self.x, self.hidden_events_sign, Color.EVENTS)
             index += 1
 
-        # Show repeated user events:
-        for event in self.repeated_user_events.items:
-            if index < self.y_cell - 1:
-                user_event_view = UserEventView(self.stdscr, self.y + index, self.x, event, self.screen)
-                user_event_view.render()
-            else:
-                self.display_line(self.y + self.y_cell - 2, self.x, self.hidden_events_sign, Color.EVENTS)
-            index += 1
+        # Show repeated user events and events from ics:
+        for event_list in [self.repeated_user_events.items, self.user_ics_events.items]:
+            for event in event_list:
+                if index < self.y_cell - 1:
+                    user_event_view = UserEventView(self.stdscr, self.y + index, self.x, event, self.screen)
+                    user_event_view.render()
+                else:
+                    self.display_line(self.y + self.y_cell - 2, self.x, self.hidden_events_sign, Color.EVENTS)
+                index += 1
 
-        # Show deadlines for tasks:
-        for event in self.deadlines.items:
-            if index < self.y_cell - 1:
-                deadline_view = DeadlineView(self.stdscr, self.y + index, self.x, event, self.screen)
-                deadline_view.render()
-            else:
-                self.display_line(self.y + self.y_cell - 2, self.x, self.hidden_events_sign, Color.DEADLINES)
-            index += 1
+        # Show deadlines for tasks, both from csv and ics files:
+        for deadline_list in [self.deadlines.items, self.deadlines_ics.items]:
+            for event in deadline_list:
+                if index < self.y_cell - 1:
+                    deadline_view = DeadlineView(self.stdscr, self.y + index, self.x, event, self.screen)
+                    deadline_view.render()
+                else:
+                    self.display_line(self.y + self.y_cell - 2, self.x, self.hidden_events_sign, Color.DEADLINES)
+                index += 1
 
         # Show holidays:
         if not cf.DISPLAY_HOLIDAYS:
@@ -466,10 +446,6 @@ class DailyView(View):
                 self.display_line(self.y + self.y_cell - 2, self.x, self.hidden_events_sign, Color.BIRTHDAYS)
             index += 1
 
-        # if index == 0 and self.screen.calendar_state == CalState.DAILY and cf.SHOW_NOTHING_PLANNED:
-            # self.display_line(self.y, self.x, MSG_TS_NOTHING, Color.UNIMPORTANT)
-            # index += 1
-
         self.num_events_this_day = index
 
 
@@ -523,6 +499,7 @@ class HeaderView(View):
         self.screen = screen
 
     def render(self):
+        """Render this view on the screen"""
         _, x_max = self.stdscr.getmaxyx()
 
         # Show title:
@@ -552,6 +529,7 @@ class FooterView(View):
         self.screen = screen
 
     def render(self):
+        """Render this view on the screen"""
         if not cf.SHOW_KEYBINDINGS: return
         clear_line(self.stdscr, self.screen.y_max - 1)
         if self.screen.state == AppState.CALENDAR:
@@ -572,6 +550,7 @@ class SeparatorView(View):
         self.screen = screen
 
     def render(self):
+        """Render this view on the screen"""
         _, x_max = self.stdscr.getmaxyx()
         x_separator = x_max - self.screen.journal_pane_width
         y_cell = (self.screen.y_max - 3) // 6
@@ -593,6 +572,7 @@ class CalenarBoarderView(View):
         self.screen = screen
 
     def render(self):
+        """Render this view on the screen"""
         x_cell = self.screen.x_max // 7
         y_cell = (self.screen.y_max - 3) // 6
 
@@ -622,6 +602,7 @@ class DaysNameView(View):
         self.screen = screen
 
     def render(self):
+        """Render this view on the screen"""
         num = 2 if self.screen.x_max < 74 else 12
         x_cell = int(self.screen.x_max // 7)
 
@@ -644,13 +625,15 @@ class DaysNameView(View):
 class DailyScreenView(View):
     """Daily view showing events of the day"""
 
-    def __init__(self, stdscr, y, x, weather, user_events, holidays, birthdays, user_tasks, screen):
+    def __init__(self, stdscr, y, x, weather, user_events, user_ics_events, holidays, birthdays, user_tasks, user_ics_tasks, screen):
         super().__init__(stdscr, y, x)
         self.weather = weather
         self.user_events = user_events
+        self.user_ics_events = user_ics_events
         self.holidays = holidays
         self.birthdays = birthdays
         self.user_tasks = user_tasks
+        self.user_ics_tasks = user_ics_tasks
         self.screen = screen
 
     @property
@@ -683,8 +666,10 @@ class DailyScreenView(View):
         return cf.TODAY_ICON if self.screen.date == self.screen.today else ''
 
     def render(self):
+        """Render this view on the screen"""
         self.screen.state = AppState.CALENDAR
-        if self.screen.x_max < 6 or self.screen.y_max < 3: return
+        if self.screen.x_max < 6 or self.screen.y_max < 3:
+            return
         # self.fill_background()
         curses.halfdelay(255)
 
@@ -709,7 +694,8 @@ class DailyScreenView(View):
             self.display_line(self.y + 2 + vertical_shift, self.x, day_string, self.color)
 
             daily_view = DailyView(self.stdscr, self.y + 3 + vertical_shift, self.x, repeated_user_events,
-                        self.user_events, self.holidays, self.birthdays, self.user_tasks, self.screen, 0, is_selection_day)
+                                   self.user_events, self.user_ics_events, self.holidays, self.birthdays,
+                                   self.user_tasks, self.user_ics_tasks, self.screen, 0, is_selection_day)
             daily_view.render()
             vertical_shift += daily_view.num_events_this_day + 3
             self.screen.next_day()
@@ -723,16 +709,19 @@ class DailyScreenView(View):
 class MonthlyScreenView(View):
     """Monthly view showing events of the month"""
 
-    def __init__(self, stdscr, y, x, weather, user_events, holidays, birthdays, user_tasks, screen):
+    def __init__(self, stdscr, y, x, weather, user_events, user_ics_events, holidays, birthdays, user_tasks, user_ics_tasks, screen):
         super().__init__(stdscr, y, x)
         self.weather = weather
         self.user_events = user_events
+        self.user_ics_events = user_ics_events
         self.holidays = holidays
         self.birthdays = birthdays
         self.user_tasks = user_tasks
+        self.user_ics_tasks = user_ics_tasks
         self.screen = screen
 
     def render(self):
+        """Render this view on the screen"""
         self.screen.state = AppState.CALENDAR
         if self.screen.x_max < 6 or self.screen.y_max < 3: return
         curses.halfdelay(255)
@@ -763,8 +752,9 @@ class MonthlyScreenView(View):
 
                     # Display the events:
                     self.screen.day = day
-                    daily_view = DailyView(self.stdscr, 3 + row * y_cell, col * x_cell, repeated_user_events, self.user_events,
-                                            self.holidays, self.birthdays, self.user_tasks, self.screen, num_events_this_month)
+                    daily_view = DailyView(self.stdscr, 3 + row * y_cell, col * x_cell, repeated_user_events,
+                                           self.user_events, self.user_ics_events, self.holidays, self.birthdays,
+                                           self.user_tasks, self.user_ics_tasks, self.screen, num_events_this_month)
                     daily_view.render()
                     num_events_this_month += len(self.user_events.filter_events_that_day(self.screen).items)
 
@@ -774,10 +764,11 @@ class MonthlyScreenView(View):
 
 
 class JournalScreenView(View):
-    def __init__(self, stdscr, y, x, weather, user_tasks, screen):
+    def __init__(self, stdscr, y, x, weather, user_tasks, user_ics_tasks, screen):
         super().__init__(stdscr, y, x)
         self.weather = weather
         self.user_tasks = user_tasks
+        self.user_ics_tasks = user_ics_tasks
         self.screen = screen
         self.refresh_time = 255
 
@@ -805,7 +796,7 @@ class JournalScreenView(View):
         header_view.render()
 
         # Display the tasks:
-        journal_view = JournalView(self.stdscr, 2, self.screen.x_min, self.user_tasks, self.screen)
+        journal_view = JournalView(self.stdscr, 2, self.screen.x_min, self.user_tasks, self.user_ics_tasks, self.screen)
         journal_view.render()
 
 
@@ -917,14 +908,15 @@ def main(stdscr) -> None:
     if cf.SHOW_WEATHER:
         print(MSG_WEATHER)
         weather.load_from_wttr()
-    screen = Screen(stdscr, cf.PRIVACY_MODE, cf.DEFAULT_VIEW, cf.SPLIT_SCREEN, cf.RIGHT_PANE_PERCENTAGE, cf.USE_PERSIAN_CALENDAR, cf.DEFAULT_CALENDAR_VIEW)
-    file_repository = FileRepository(cf.TASKS_FILE, cf.EVENTS_FILE, cf.HOLIDAY_COUNTRY, cf.USE_PERSIAN_CALENDAR)
+    screen = Screen(stdscr, cf)
+    file_repository = FileRepository(cf)
     user_events = file_repository.load_events_from_csv()
     user_tasks = file_repository.load_tasks_from_csv()
+    user_ics_tasks = file_repository.load_tasks_from_ics()
+    user_ics_events = file_repository.load_events_from_ics()
     holidays = file_repository.load_holidays()
     birthdays = file_repository.load_birthdays_from_abook()
-    importer = Importer(user_tasks, user_events, cf.TASKS_FILE, cf.EVENTS_FILE, cf.CALCURSE_TODO_FILE,
-                                cf.CALCURSE_EVENTS_FILE, cf.TASKWARRIOR_FOLDER, cf.USE_PERSIAN_CALENDAR)
+    importer = Importer(user_tasks, user_events, cf)
 
     read_items_from_user_arguments(screen, user_tasks, user_events, file_repository)
 
@@ -932,13 +924,15 @@ def main(stdscr) -> None:
     stdscr = curses.initscr()
     curses.noecho()
     curses.curs_set(False)
-    initialize_colors()
+    initialize_colors(cf)
 
     # Initialise screen views:
     app_view = View(stdscr, 0, 0)
-    monthly_screen_view = MonthlyScreenView(stdscr, 0, 0, weather, user_events, holidays, birthdays, user_tasks, screen)
-    daily_screen_view = DailyScreenView(stdscr, 0, 0, weather, user_events, holidays, birthdays, user_tasks, screen)
-    journal_screen_view = JournalScreenView(stdscr, 0, 0, weather, user_tasks, screen)
+    monthly_screen_view = MonthlyScreenView(stdscr, 0, 0, weather, user_events, user_ics_events,
+                                            holidays, birthdays, user_tasks, user_ics_tasks, screen)
+    daily_screen_view = DailyScreenView(stdscr, 0, 0, weather, user_events, user_ics_events,
+                                        holidays, birthdays, user_tasks, user_ics_tasks, screen)
+    journal_screen_view = JournalScreenView(stdscr, 0, 0, weather, user_tasks, user_ics_tasks, screen)
     help_screen_view = HelpScreenView(stdscr, 0, 0, screen)
     welcome_screen_view = WelcomeScreenView(stdscr, 0, 0, screen)
     footer_view = FooterView(stdscr, 0, 0, screen)
