@@ -9,6 +9,7 @@ import getopt
 from calcure.data import AppState
 
 
+
 class Config:
     """User configuration loaded from the config.ini file"""
     def __init__(self):
@@ -65,7 +66,7 @@ class Config:
                 "privacy_icon":              "•",
                 "today_icon":                "•",
                 "birthday_icon":             "★",
-                "holiday_icon":              "☘️",
+                "holiday_icon":              "⛱",
                 "hidden_icon":               "...",
                 "done_icon":                 "✔",
                 "todo_icon":                 "•",
@@ -220,6 +221,15 @@ class Config:
             self.RIGHT_PANE_PERCENTAGE = int(conf.get("Parameters", "right_pane_percentage", fallback=25))
             self.ONE_TIMER_AT_A_TIME   = conf.getboolean("Parameters", "one_timer_at_a_time", fallback=False)
 
+            # ICS files:
+            self.ICS_EVENT_FILES = conf.get("Parameters", "ics_event_files", fallback=None)
+            if self.ICS_EVENT_FILES is not None:
+                self.ICS_EVENT_FILES = [str(i) for i in self.ICS_EVENT_FILES.split(",")]
+
+            self.ICS_TASK_FILES = conf.get("Parameters", "ics_task_files", fallback=None)
+            if self.ICS_TASK_FILES is not None:
+                self.ICS_TASK_FILES = [str(i) for i in self.ICS_TASK_FILES.split(",")]
+
             # Calendar colors:
             self.COLOR_TODAY           = int(conf.get("Colors", "color_today", fallback=2))
             self.COLOR_EVENTS          = int(conf.get("Colors", "color_events", fallback=4))
