@@ -4,44 +4,44 @@ import configparser
 import pathlib
 import csv
 import os
-
 import datetime
-import jdatetime
+
+from ics import Calendar as IcsCalendar
 
 from calcure.data import *
+from calcure.helpers import convert_to_persian_date, convert_to_gregorian_date
 
 
-def convert_to_persian_date(year, month, day):
-    """Convert date from Gregorian to Persian calendar"""
-    persian_date =  jdatetime.date.fromgregorian(day=day, month=month, year=year)
-    day = persian_date.day
-    month = persian_date.month
-    year = persian_date.year
-    return year, month, day
-
-
-def convert_to_gregorian_date(year, month, day):
-    """Convert date from Persian to Gregorian calendar"""
-    gregorian_date = jdatetime.date(year, month, day).togregorian()
-    day = gregorian_date.day
-    month = gregorian_date.month
-    year = gregorian_date.year
-    return year, month, day
+def clean_ics_file(filename):
+    """Remove multiple PRODID lines from ics file"""
+    original_file = filename
+    clean_file = filename + ".clean"
+    previous_line = ""
+    with open(original_file, 'r') as input_file, open(clean_file, 'w') as output_file:
+        for line in input_file:
+            # If there is more than one PRODID line, skip them:
+            if not ("PRODID:" in line and "PRODID:" in previous_line):
+                output_file.write(line)
+            previous_line = line
 
 
 class FileRepository:
     """Load and save events and tasks to files"""
 
-    def __init__(self, tasks_file, events_file, country, use_persian_calendar):
+    def __init__(self, cf):
         self.user_tasks = Tasks()
         self.user_events = Events()
         self.holidays = Events()
         self.birthdays = Birthdays()
+        self.user_ics_tasks = Tasks()
+        self.user_ics_events = Events()
         self.abook_file = str(pathlib.Path.home())+"/.abook/addressbook"
-        self.tasks_file = tasks_file
-        self.events_file = events_file
-        self.country = country
-        self.use_persian_calendar = use_persian_calendar
+        self.tasks_file = cf.TASKS_FILE
+        self.events_file = cf.EVENTS_FILE
+        self.country = cf.HOLIDAY_COUNTRY
+        self.use_persian_calendar = cf.USE_PERSIAN_CALENDAR
+        self.ics_event_files = cf.ICS_EVENT_FILES
+        self.ics_task_files = cf.ICS_TASK_FILES
 
     @property
     def is_task_format_old(self):
@@ -52,6 +52,7 @@ class FileRepository:
 
     def read_or_create_file(self, file):
         """Read user's csv file or create new one if it does not exist"""
+
         # Try to read the file line by line:
         try:
             with open(file, "r", encoding="utf-8") as f:
@@ -94,17 +95,21 @@ class FileRepository:
             # Read task name and statuses:
             if row[0 + shift][0] == '.':
                 name = row[0 + shift][1:]
-                privacy = True
+                is_private = True
             else:
                 name = row[0 + shift]
-                privacy = False
+                is_private = False
             status = Status[row[1 + shift].upper()]
             stamps = row[(2 + shift):] if len(row) > 2 else []
-            self.user_tasks.add_item(Task(task_id, name, status, Timer(stamps), privacy, year, month, day))
+            timer = Timer(stamps)
+
+            # Add task:
+            new_task = Task(task_id, name, status, timer, is_private, year, month, day)
+            self.user_tasks.add_item(new_task)
         return self.user_tasks
 
     def load_events_from_csv(self):
-        """Reads from user's file or create it if it does not exist"""
+        """Reads from user's csv file or create it if it does not exist"""
         lines = self.read_or_create_file(self.events_file)
         for index, row in enumerate(lines):
             event_id = index
@@ -113,10 +118,10 @@ class FileRepository:
             day = int(row[3])
             if row[4][0] == '.':
                 name = row[4][1:]
-                privacy = True
+                is_private = True
             else:
                 name = row[4]
-                privacy = False
+                is_private = False
 
             # Account for old versions of the datafile:
             if len(row) > 5:
@@ -146,8 +151,9 @@ class FileRepository:
             if self.use_persian_calendar:
                 year, month, day = convert_to_persian_date(year, month, day)
 
-            self.user_events.add_item(UserEvent(event_id, year, month, day,
-                                name, repetition, frequency, status, privacy))
+            # Add event:
+            new_event = UserEvent(event_id, year, month, day, name, repetition, frequency, status, is_private)
+            self.user_events.add_item(new_event)
         return self.user_events
 
     def save_tasks_to_csv(self):
@@ -176,7 +182,7 @@ class FileRepository:
         """Rewrite the data file with changed events"""
         original_file = self.events_file
         dummy_file = self.events_file + '.bak'
-        with open(dummy_file, "w", encoding="utf-8") as f:
+        with open(dummy_file, "w", encoding="utf-8") as file:
             for ev in self.user_events.items:
 
                 # If persian calendar was used, we convert event back to Gregorian for storage:
@@ -186,23 +192,17 @@ class FileRepository:
                     year, month, day = ev.year, ev. month, ev.day
 
                 name = f'{"."*ev.privacy}{ev.name}'
-                f.write(f'{ev.item_id},{year},{month},{day},"{name}",{ev.repetition},{ev.frequency.name.lower()},{ev.status.name.lower()}\n')
+                file.write(f'{ev.item_id},{year},{month},{day},"{name}",{ev.repetition},{ev.frequency.name.lower()},{ev.status.name.lower()}\n')
         os.remove(original_file)
         os.rename(dummy_file, original_file)
         self.user_events.changed = False
 
-    def load_deadlines(self):
-        """Create collection of events that are deadlines for tasks"""
-        for task in self.user_tasks.items:
-            self.deadlines.add_item(DeadlineEvent(task.item_id, task.year, task.month, task.day, task.name, task.status, task.privacy))
-        return self.deadlines
-
     def load_holidays(self):
         """Load list of holidays in this country around this year"""
         try:
             import holidays as hl
             year = datetime.date.today().year
-            holiday_events = eval("hl."+self.country+"(years=[year-1, year, year+3])")
+            holiday_events = eval("hl."+self.country+"(years=[year-2, year-1, year, year+1, year+2, year+3, year+4])")
             for date, name in holiday_events.items():
 
                 # Convert to persian date if needed:
@@ -211,13 +211,20 @@ class FileRepository:
                 else:
                     year, month, day = date.year, date.month, date.day
 
-                self.holidays.add_item(Event(year, month, day, name))
+                # Add holiday:
+                holiday = Event(year, month, day, name)
+                self.holidays.add_item(holiday)
         except (ModuleNotFoundError, SyntaxError, AttributeError):
             pass
         return self.holidays
 
     def load_birthdays_from_abook(self):
         """Loading birthdays from abook contacts"""
+
+        # Quit if file does not exists:
+        if not os.path.exists(self.abook_file):
+            return self.birthdays
+
         abook = configparser.ConfigParser()
         abook.read(self.abook_file)
         for each_contact in abook.sections():
@@ -231,86 +238,113 @@ class FileRepository:
                     if self.use_persian_calendar:
                         _, month, day = convert_to_persian_date(1000, month, day)
 
-                    self.birthdays.add_item(Event(1, month, day, name))
+                    # Add birthday:
+                    birthday = Event(1, month, day, name)
+                    self.birthdays.add_item(birthday)
         return self.birthdays
 
+    def load_tasks_from_ics(self):
+        """Load tasks from each of the ics files"""
 
-class Importer:
-    """Import tasks and events from files of other programs"""
-    def __init__(self, user_tasks, user_events, tasks_file, events_file, calcurse_todo_file,
-                            calcurse_events_file, taskwarrior_folder, use_persian_calendar):
-        self.user_tasks = user_tasks
-        self.user_events = user_events
-        self.tasks_file = tasks_file
-        self.events_file = events_file
-        self.calcurse_todo_file = calcurse_todo_file
-        self.calcurse_events_file = calcurse_events_file
-        self.taskwarrior_folder = taskwarrior_folder
-        self.use_persian_calendar = use_persian_calendar
+        # Quit if the file is not specified:
+        if self.ics_task_files is None:
+            return self.user_ics_tasks
 
-    def read_file(self, file):
-        """Try to read a file and return its lines"""
-        try:
-            with open(file, "r", encoding="utf-8") as f:
-                lines = f.readlines()
-            return lines
-        except (IOError, FileNotFoundError, NameError):
-            return []
+        for filename in self.ics_task_files:
 
-    def import_tasks_from_calcurse(self):
-        """Import tasks from calcurse database"""
-        lines = self.read_file(self.calcurse_todo_file)
-        for line in lines:
-            name = line[4:-1]
-            importance = line[1]
-            if (len(name) > 0) and not self.user_tasks.item_exists(name):
-                if importance in ['1', '2']:
-                    status = Status.IMPORTANT
-                elif importance in ['8', '9', '10']:
-                    status = Status.UNIMPORTANT
-                else:
+            # Quit if file does not exists:
+            if not os.path.exists(filename):
+                return self.user_ics_tasks
+
+            # Clean and read the file, then remove tmp file:
+            clean_ics_file(filename)
+            with open(filename + ".clean", 'r', encoding="utf-8") as file:
+                ics_text = file.read()
+            os.remove(filename + ".clean")
+
+            cal = IcsCalendar(ics_text)
+            for task in cal.todos:
+                if task.status != "CANCELLED":
+                    task_id = self.user_ics_tasks.generate_id()
+
+                    # Assign status from priority:
                     status = Status.NORMAL
-                task_id = self.user_tasks.generate_id()
-                privacy = False
-                self.user_tasks.add_item(Task(task_id, name, status, Timer([]), privacy))
+                    if task.priority is not None:
+                        if task.priority > 5:
+                            status = Status.UNIMPORTANT
+                        if task.priority < 5:
+                            status = Status.IMPORTANT
 
-    def import_tasks_from_taskwarrior(self):
-        """Import tasks from taskwarrior database"""
-        lines = self.read_file(self.taskwarrior_folder+"/pending.data")
-        for line in lines:
-            if len(line) > 0:
-                name = line.split('description:"', 1)[1]
-                name = name.split('"', 1)[0]
-                if not self.user_tasks.item_exists(name):
-                    task_id = self.user_tasks.generate_id()
-                    privacy = False
-                    self.user_tasks.add_item(Task(task_id, name, Status.NORMAL, Timer([]), privacy))
+                    # Correct according to status:
+                    if task.status == "COMPLETED":
+                        status = Status.DONE
 
-    def import_events_from_calcurse(self):
-        """Importing events from calcurse apt file into our events file"""
-        lines = self.read_file(self.calcurse_events_file)
-        for line in lines:
-            month = int(line[0:2])
-            day = int(line[3:5])
-            year = int(line[6:10])
-            if line[11] == "[":
-                name = line[15:-1]
-            elif line[11] == "@":
-                name = line[35:-1]
-                name = name.replace('|',' ')
-            else:
-                name = ''
-            if not self.user_events.items:
-                event_id = 0
-            else:
-                event_id = self.user_events.items[-1].item_id + 1
-            privacy = False
+                    name = task.name
 
-            # Convert to persian date if needed:
-            if self.use_persian_calendar:
-                year, month, day = convert_to_persian_date(year, month, day)
+                    # Try reading task due date:
+                    try:
+                        year = task.due.year
+                        month = task.due.month
+                        day = task.due.day
+                    except AttributeError:
+                        year, month, day = 0, 0, 0
 
-            imported_event = UserEvent(event_id, year, month, day, name, 1,
-                                       Frequency.ONCE, Status.NORMAL, privacy)
-            if not self.user_events.event_exists(imported_event):
-                self.user_events.add_item(imported_event)
+                    timer = Timer([])
+                    is_private = False
+
+                    # Add task:
+                    new_task = Task(task_id, name, status, timer, is_private, year, month, day)
+                    self.user_ics_tasks.add_item(new_task)
+
+        return self.user_ics_tasks
+
+    def load_events_from_ics(self):
+        """Load events from each of the ics files"""
+
+        # Quit if the file is not specified:
+        if self.ics_event_files is None:
+            return self.user_ics_events
+
+        for filename in self.ics_event_files:
+
+            # Quit if file does not exists:
+            if not os.path.exists(filename):
+                return self.user_ics_events
+
+            # Clean and read the file, then remove tmp file:
+            clean_ics_file(filename)
+            with open(filename + ".clean", 'r', encoding="utf-8") as file:
+                ics_text = file.read()
+            os.remove(filename + ".clean")
+            cal = IcsCalendar(ics_text)
+            for index, event in enumerate(cal.events):
+
+                # Default parameters:
+                event_id = index
+                repetition = '1'
+                frequency = Frequency.ONCE
+                status = Status.NORMAL
+                is_private = False
+
+                # Parameters of the event from ics if they exist:
+                name = event.name if event.name is not None else ""
+                all_day = event.all_day if event.all_day is not None else True
+                year = event.begin.year if event.begin else 0
+                month = event.begin.month if event.begin else 1
+                day = event.begin.day if event.begin else 1
+
+                # Add start time to name of non-all-day events:
+                if not all_day:
+                    hour = event.begin.hour if event.begin else 0
+                    minute = event.begin.minute if event.begin else 0
+                    name = f"{hour:0=2}:{minute:0=2} {name}"
+
+                # Convert to persian date if needed:
+                if self.use_persian_calendar:
+                    year, month, day = convert_to_persian_date(year, month, day)
+
+                # Add event:
+                new_event = UserEvent(event_id, year, month, day, name, repetition, frequency, status, is_private)
+                self.user_ics_events.add_item(new_event)
+
+        return self.user_ics_events
